"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
function useAsync(asyncFn, args, deps) {
    if (!deps)
        deps = args;
    const [state, setState] = (0, react_1.useState)({
        data: null,
        error: undefined,
        pending: false
    });
    const [counter, setCounter] = (0, react_1.useState)(0);
    (0, react_1.useEffect)(() => {
        let mounted = true;
        if (!state.pending)
            updateState({ pending: true });
        const onSuccess = (data) => mounted && updateState({ data, pending: false, error: null });
        const onError = (error) => mounted && updateState({ error, pending: false, data: null });
        const promise = asyncFn(...args);
        promise.then(onSuccess);
        promise.catch(onError);
        return () => { mounted = false; };
    }, [...deps, counter]);
    return { ...state, refresh };
    function refresh() {
        if (state.pending)
            return;
        setCounter(counter => counter + 1);
    }
    function updateState(nextState) {
        setState(state => ({ ...state, ...nextState }));
    }
}
exports.default = useAsync;
//# sourceMappingURL=use-async.js.map